#!/usr/bin/perl

###############################################################################
# $Id: loguploadclient.pl 214 2011-11-30 00:11:52Z david $
#
# Copyright(c) 2006-2011 ThreatSTOP, Inc. All Rights Reserved
# 
# NOTICE:  All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
# technical concepts contained herein are proprietary to ThreatSTOP, Inc.
# and its suppliers and may be covered by U.S. and Foreign Patents, patents
# in process, and are protected by trade secret or copyright law.
# Dissemination of this information or reproduction of this material is
# strictly forbidden unless prior written permission is obtained from
# ThreatSTOP, Inc.
#
#
# ThreatSTOP log upload script
my $VERSION = '1.14';
# file: loguploadclient.pl
# takes no mandatory arguments. logfile and url to upload in threatstop.conf
# outputs messages to /tmp/ts-logupload.txt
#  v 1.10 Fix to split log into smaller parts before uploading
#  v 1.11 Remove exit statement in middle, replace with qx('logger result')
#  v 1.12 If ip address is specified as argument set the fw_ip field in the
#         upload form
#  v 1.13 Renamed output file to ts-logupload.txt. Cleaned up format 
#  v 1.14 Allow command line logfile parm to overwrite logfile in threatstop.conf
#        Note this ONLY works if IP address is specified i.e.
#        loguploadclient.pl deviceip logfilename
###############################################################################
use strict;
use LWP::UserAgent;
use HTTP::Request::Common;
use Digest::MD5;

my %cfg;
my $path = substr( $0, 0, rindex( $0, '/' ) + 1 );
my $t = gmtime;
open( OUT, '>/tmp/ts-logupload.txt' );
print OUT "$0 started at $t\n";
open( FILE, '<' . $path . 'threatstop.conf' )
  or die "can't find threatstop config file";
  
while (<FILE>) {
	#print $_;
	next unless (/^([a-z]\w+)=[("]?(.*?)[")]?\s+$/);
	$cfg{$1} = $2;
}
close FILE;

if (@ARGV) {
	my $ip;
	$ip = shift;
	unless ( $ip =~ /^\d+\.\d+\.\d+\.\d+$/ ) {
		print OUT "$ip not a valid ipaddress";
		exit;
	}
	$cfg{'ip'}=$ip;
}

if (@ARGV) {
	my $lf = shift;
	unless (-f $lf ) {
		print OUT "$lf not a valid logfile\n";
		exit;
	}
	$cfg{'logfile'}=$lf;
}

# start of the modifications
# we create new variables for the log file processing
my $limitlinelog = 10000;
my $tmplogfil    = '/tmp/tmplog';
my $quantity;

# read file
open( LOG, $cfg{'logfile'} );

# split log into 10 000 lines so it doesn't exceed the POST size limit
# on the webserver.
while (<LOG>) {
	++$quantity;
	if ( $quantity == 1 ) {
		open( TMP, ">$tmplogfil" ) || die "err:$!\n";
	}
	elsif ( $quantity == $limitlinelog ) {
		print TMP;
		close(TMP);
		upload();
		$quantity = 0;
		next;
	}
	print TMP;
}

close TMP;
upload();

print qx(logger "ThreatSTOP log upload complete");
close OUT;
exit;

# log file function
#upload them to server by means of &poster function and to try to send them 5 times
sub upload {
	for ( my $try = 0 ; $try < 5 ; $try++ ) {
		my $resp = poster();
		if ( $resp->is_success() ) {
			print OUT "Probably OK on $try try", $resp->content . $/;
			next unless ( $resp->content =~ /MD5: OK\s+Filesize: OK/s );
			last;
		}
		print OUT "Failed on try $try: " . $resp->as_string . $/;
	}
}

sub poster {

	# build MD5 as hexdigest
	my $local_md5 = undef;
	my $ctx       = Digest::MD5->new;

	#read file
	open( MD, $tmplogfil );

	#build md5
	my $ctx = Digest::MD5->new;
	$ctx->addfile(*MD);
	close MD;
	$local_md5 = $ctx->hexdigest;
	
	# calculate file size
	my $filesize = ( stat $tmplogfil )[7];
	# print "log size $filesize md5 $local_md5\n";

	my $ua = LWP::UserAgent->new();

	# put timeouts, proxy etc into the useragent if needed
	my $req;
	if ($cfg{'ip'}) {
		$req = POST $cfg{'url'},
		  Content_Type => 'form-data',
		  Content      => [
			submit      => 1,
			upfile_size => $filesize,
			upfile      => [$tmplogfil],
			md5_client  => $local_md5,
			fw_ip       => $cfg{'ip'}
		  ];
	}
	else {
		$req = POST $cfg{'url'},
		  Content_Type => 'form-data',
		  Content      => [
			submit      => 1,
			upfile_size => $filesize,
			upfile      => [$tmplogfil],
			md5_client  => $local_md5
		  ];
	}
	my $response = $ua->request($req);
	return $response;
}
